//**********************INCLUDES**********************************

#include "colors.inc"    // The include files contain
#include "stones.inc"    // pre-defined scene elements
#include "textures.inc"    // pre-defined scene elements
#include "shapes.inc"
#include "glass.inc"
#include "metals.inc"

//******************** CIE & LIGHTSYS ****************************
#include "LightSysIV\CIE.inc"
//CIE_ColorSystemWhitepoint(sRGB_ColSys,Illuminant_D50)
#include "LightSysIV\lightsys.inc"          
#include "LightSysIV\rspd_jvp.inc"          // predefined material samples
#include "LightSysIV\CIE.inc"
#include "LightSysIV\CIE_tools.inc"
#include "LightSysIV\lightsys_constants.inc"
#include "LightSysIV\espd_lightsys.inc"
#include "LightSysIV\lspd_elements.inc"
#include "LightSysIV\rspd_pantone_coated.inc"
#include "LightSysIV\rspd_pantone_matte.inc"
#include "LightSysIV\rspd_aster.inc"
#include "LightSysIV\CIE_Skylight"

//********************** PERSONAL INCLUDES**********************************
#include "heat-shield2.pov"



//********************** GLOBAL SETTINGS AND SWITCHES***********************
                        
#declare one_frame = 0;                     
#declare radiosity_on = 0;

global_settings { 
                    assumed_gamma 1
                    max_trace_level 5
                    ambient_light <0,0,0>*0
                    adc_bailout 0.02
  
                    #if (radiosity_on = 1) 
                    radiosity {
                                        
                                 //brightness 2.2
                                 count 50
                                 error_bound 0.15
                                 gray_threshold 0.0
                                 low_error_factor 0.2
                                 minimum_reuse 0.015
                                 nearest_count 10
                                 recursion_limit 3
                                 adc_bailout 0.02
                                 max_sample -1.0
                                 media off
                                 normal off
                                 always_sample off
                                
                                 pretrace_start 0.08
                                 pretrace_end 0.01
                    
                            } 
                    #end
 
  
  
                }
                
                   
//********************** LIGHTS & ARTIFICIAL & SUN *******************************
// to be used if probe lands on the night side, unused at the moment

              
light_source{
            <0,20,-50>*10
            //<20,0,50>*10
            Light_Color(SunColor,2)
            adaptive 0
            jitter
            }

#declare LC3 = EmissiveSpectrum(ES_HS_Sodium_Lamp);
#declare LC4 = EmissiveSpectrum(ES_Mitsubishi_Metal_Halide);
#declare LM3 = Lm_GE_LinearR7S_500w;   
   
//#declare probe_projector = Light_Color(LC3,LM3*10);

//-----------------------------------ALIEN MEDUSA MACRO-------------------------
// to be used on surface to show an alien lifeform
 
#macro alien_medusa(pos,size_rel,color_base_emi,color_base_xmit,trsp) 

sphere { pos, size_rel 

         hollow
         interior {
	                media{
	                        emission rgb<0.2,0.1,0.8*color_base_emi>*0.02/size_rel*10*color_base_emi
	                        //scattering{ 1, 0.17 exnction 0.01}
	                        method 3
	                        intervals 5
	                        samples 1,1
	                        
	                        density{ 
	                                 spherical
                                     turbulence 0
                                     color_map {
                                                 [0 rgb <0.2,0.1,0.8>*5/size_rel*10*color_base_emi]//border
                                                 [1 rgb <0.5,0,0.2>*2/size_rel*10*color_base_emi]//center
	                                           }
	                    
	                               }
	                     }                    
	                                                                    
	             } 

	          

         texture { 
                    pigment{ crackle scale 0.1
                             color_map{[0.0 color rgbt <0,0,0.7,1*trsp>*1.3*color_base_xmit]
                                       [0.5 color rgbt <0.5,0,0.75,0.7*trsp>*1.15*color_base_xmit]
                                       [0.5 color rgbt <0.2,0.5,0.8,0.3*trsp>*1.25*color_base_xmit]
                                       [1.0 color rgbt <0.2,0,2,0.3*trsp>*1.3*color_base_xmit]
                   
                                      }//end of color_map
                           } // end of pigment}
                    normal { 
                             waves 1.5 frequency 0.5 scale 0.02
                           }
                           
                    finish {
                             ambient 0.01 diffuse 0.5 phong 0.5 crand 2.3 reflection{ 0.6 metallic 0.85}
                            //irid { 0.2 thickness 0.1 } 
                           }
                 } // end of texture 

         scale<1,1,1>  rotate<0,0,0>  //translate<0,0.5,0>

       } // end of sphere 
#end
    
     /* 
     alien_medusa(<0,0,0>,2.55,1,1.2,0.5)
     alien_medusa(<0,0,4.5>,1.25,1,1.6,0.4)
     alien_medusa(<4.5,0,0>,1.25,1,1.5,0.35)
     alien_medusa(<-4.5,0,4.5>,1.25,1,1.8,0.3)
     alien_medusa(<4.5,0,-4.5>,1.25,1,2,0.3)
       */                                       
                                              
//-----------------------------------OCEAN FUNCTIONS-------------------------

#declare fn_pattern=
                    function {
                                pattern { bozo turbulence 2 noise_generator 1.2 scale <1,1,1>*0.7 }
                             }


#declare fn_water=
                    function {
                                sqrt(pow(x,2) + pow(y,2) + pow(z,2)) - 0.04*((fn_pattern(x,y,z))) - 4.47
                             } 
                                              
//-----------------------------------TERRAIN FUNCTIONS-------------------------

#declare fn_pattern2=
                    function {
                                pattern { wrinkles turbulence 4 noise_generator 2.2 scale <1,1,1>*0.7 }
                             }
                                      
   
#declare fn_terrain=                         
                    function {
                                sqrt(pow(x,2) + pow(y,2) + pow(z,2)) - abs(0.07*((fn_pattern2(x,y,2*z)))) - 4.478
                             } 
   
//-----------------------------------OCEAN DECLARATION-------------------------  
  
#declare ocean = isosurface {
                                function { fn_water(x, y, z) }
                                accuracy 0.05
                                max_gradient 5                                       
                                contained_by { sphere { <0,0,0>,5.2 }}
                                
                                texture {      
                                          pigment { color rgbt <0.0,0.3,0.6,0.4> }

				                          finish {ambient 0.0 reflection 0.4
				                                  //irid { 0.6 thickness 0.4 } 
				                                 }

			                            }				
                            }
    

//-----------------------------------TERRAIN DECLARATION-------------------------    
    
#declare terrain = isosurface {
                                function { fn_terrain(x, y, z) }
                                accuracy 0.05
                                max_gradient 2.5
                                contained_by { sphere { <0,0,0>,5.25 }}
                                
                                texture {
                                          pigment {color rgb < 0.8, 0.6, 0.15>}
                                          normal {granite 0.2 scale 0.02}
                                          normal { bumps 0.4 scale 0.02}
 
                                          finish {
                                                   brilliance 1.6
                                                   specular 0.3
                                                   ambient 0.0
                                                 }
                                        }				
                             }
 
//-----------------------------------ATMOSPHERE DECLARATION (MEDIA)--------------
   
declare atmosphere = sphere {
                              <0,0,0>,1
                              hollow 
     
                              material {
                                         texture {
                                                    pigment { rgbt <1,1,1,0.75> }
                                                    //finish {irid { 2.6 thickness 0.4 }}
                                                 } 
     
                                         interior {
                                                    media {
                                                            method 3
                                                            intervals 1
                                                            confidence 0.0001
                                                            variance 1.0/1.0
                                                            scattering {4,<0,0.502,1.0>*2}
                                                            emission <0,0.502,1.0>*0.1
                                                            density {
                                                                      spherical
                                                                      color_map {  
                                                                                  [ 0 rgb <0.000,0.000,0.000>*0.01 ]
                                                                                  [ 0.0005 rgb <0.001,0.001,0.001>*0.1 ]
                                                                                  [ 0.001 rgb <0.001,0.001,0.001>*0.5 ]
                                                                                  [ 0.005 rgb <0.001,0.001,0.001>*0.7 ]
                                                                                  [ 0.1 rgb 0.08*1.75 ]
                                                                                  [ 0.2 rgb 0.1*1.5 ]              
                                                                                  [ 0.3 rgb 0.5*0.75 ]
                                                                                  [ 1.0 rgb 0 ]
                                                                                  /*
                                                                                  [ 0 rgb <0.000,0.000,0.000>*0.1 ]
                                                                                  [ 0.0005 rgb <0.001,0.001,0.001>*0.5 ]
                                                                                  [ 0.001 rgb <0.001,0.001,0.001>*1 ]
                                                                                  [ 0.005 rgb <0.001,0.001,0.001>*6 ]
                                                                                  [ 0.1 rgb 0.08*4.5 ]
                                                                                  [ 0.2 rgb 0.1*3 ]              
                                                                                  [ 0.3 rgb 0.5*1.5 ]
                                                                                  [ 1.0 rgb 0 ]
                                                                                  */
                                                                                }
                                                                    }

                                                          }
                                                  }
  
                                        }  
     
                              scale 5.3  
                            } 

     
//------------------------CLOUDS LAYERS---------------------------------------------------
      
 #declare CloudTexture = texture {
                                  pigment { 
                                            bozo
                                            colour_map {
                                                          [0.00 colour rgbt <0,0,0,1>]
                                                          [0.40 colour rgbt <1,1,1,1>]
                                                          [0.60 colour rgbt <1,1,1,0.80>] 
                                                          [0.85 colour rgbt <1,1,1,0>]
                                                          [1.00 colour rgbt <1,1,1,0>]
                                                       }
                                            turbulence 4.0
                                            lambda 3.0           
                                            omega 0.55
                                            ramp_wave
                                            scale <1,0.55,0.5>*5 
                                             
                                          }
                                 } 

#declare CloudTexture2 = texture {
                                  pigment { 
                                            bozo
                                            colour_map {
                                                          [0.00 colour rgbt <0,0,0,1>]
                                                          [0.55 colour rgbt <1,1,1,1>] 
                                                          [0.65 colour rgbt <1,1,1,0.70>]
                                                          [0.80 colour rgbt <1,1,1,0>]
                                                          [1.00 colour rgbt <1,1,1,0>]
                                                       }
                                            turbulence 2.0
                                            lambda 3.0
                                            omega 0.55
                                            ramp_wave
                                            scale <1,0.9,0.5>*5 
                                         }
                                } 


#declare CloudSphere1 = sphere { <0,0,0>,4.65 
                                    texture { CloudTexture }   
                                    hollow
                               }
                               
#declare Cloudsphere2 = sphere { <0,0,0>, 4.72
                                    texture { CloudTexture2 }  
                                    hollow
                               }

#declare PlanetCloudSphere0 = union {  
                                      object {CloudSphere1}
                                      object {Cloudsphere2}
                                    }                           
                                    
                                    
//---------------------------PLANET INSTANTIATION------------------------------------------
 
object {PlanetCloudSphere0}
object {terrain}
object {ocean}
object {atmosphere}

//---------------------------ORBIT PARAMETERS AND CAMERA VECTORS---------------------------
 
#declare max_items = 1;
#declare amt = array[1];
//#declare amt2 = 0.04*(1.4*exp(sin(0.025*((clock + 1/25) - 72.2))) - 3.8); // function of clock to decrease the amount of probe rotation around the planet center (PREVIEW ANIMATION)
#declare amt2 = 1*(1.4*exp(sin(0.025*((clock) - 72.2))) - 3.8); // function of clock to decrease the amount of probe rotation around the planet center (FULL LENGTH ANIMATION)


#macro SaveState()
       #fopen file "state.dat" write
       #local i = 0;
       # while (i<max_items)
               #write ( file, amt[i])
               #local i=i+1;
       #end
       #fclose file
#end

#macro LoadState()
       #fopen rfile "state.dat" read
       #local i = 0;
       #while (i<max_items)
               #read ( rfile, amt3)
               #local i=i+1;
       #end
       #fclose file
       #declare amt[0] = amt3;
#end

#if (clock = 1)
#declare amt[0] = amt2; //initialize rotation equation at animation start
SaveState()
#end
 
LoadState()

#declare altitude_function = -0.5*exp(sin(0.025*(clock - 72.2))) + 5.91; // function to decrease altitude of the probe

#declare orbit = <altitude_function,0,0>;  // vector giving altitude of the probe in thousands of Km
#declare orbit = vrotate(orbit,<-60,0,0>); // tilt orbit up
#declare orbit = vrotate(orbit,<0,30,0>); // twist orbit a bit
#declare orbit = vrotate(orbit,<0,amt[0],0>); // angular motion equation of the probe in degrees

#declare orbit2 = <altitude_function,0,0>;   // vector giving altitude of the probe in thousands of Km
#declare orbit2 = vrotate(orbit2,<-60,0,0>); // tilt orbit up
#declare orbit2 = vrotate(orbit2,<0,30,0>); // twist orbit a bit
#declare orbit2 = vrotate(orbit2,<0,amt[0] +26*(exp(sin(0.024*((clock) - 72.2)))-2.72),0>); // angular motion equation of the probe in degrees + tilt camera view towards horizon as the probe descends

#declare height = orbit.x; //height from planet center in thousands of Km
#declare height_atm = (height -5.3)*1000000; // height from upper bound of the stratosphere (edge of stratosphere media) in meters

      
#declare amt[0] = amt[0] + amt2; //integration giving the total amount of rotation of the probe around planet center since start
#if (one_frame = 0)
SaveState()
#end

#declare orbit_shield2 = <-0.5*exp(sin(0.025*(clock - 72.0))) + 5.91,0,0>;
#declare orbit_shield2 = vrotate(orbit_shield2,<-60,0,0>); // tilt orbit up
#declare orbit_shield2 = vrotate(orbit_shield2,<0,30,0>); // twist orbit a bit
#declare orbit_shield2 = vrotate(orbit_shield2,<0,amt[0] -60,0>); // rotate about origin


object {heat_shield rotate -y*180 rotate -amt[0] scale 0.00001 translate orbit translate (orbit_shield2/10000 - orbit/10000)*0.0000001*pow(clock*9,2)}                                                                              
//shield declaration and motion equation after separation (separation at beginning for the moment, in the future start at clock > 100)

#declare shieldpos = orbit - (orbit_shield2/10000 - orbit/10000)*0.0000001*pow(clock*9,2);
//vector expression to get heat shield position (duplicate from above - problem when factorizing !)

#declare surface_pos = trace(ocean,orbit,vnormalize(-orbit)*0.01); // surface distance from planet center at probe vertical (radar altimeter)
#declare prograde = orbit2 - orbit; // vector for prograde motion equation
#declare pro_right = vcross(orbit,prograde); //unused, names says all

//----- DISTANCE FORMULAS-----------------------------
#declare radar_altx = abs(orbit.x) - abs(surface_pos.x);
#declare radar_alty = abs(orbit.y) - abs(surface_pos.y);
#declare radar_altz = abs(orbit.z) - abs(surface_pos.z);

#declare dst_shieldx = abs(orbit.x) - abs(shieldpos.x);
#declare dst_shieldy = abs(orbit.y) - abs(shieldpos.y);
#declare dst_shieldz = abs(orbit.z) - abs(shieldpos.z); 

#declare radar_alt = sqrt(pow(radar_altx,2) + pow(radar_alty,2) + pow(radar_altz,2))*1000000; // radar altimeter in meters of the probe 
#declare dst_shield = sqrt(pow(dst_shieldx,2) + pow(dst_shieldy,2) + pow(dst_shieldz,2))*1000000; //distance from probe to heat_shield after separation in meters

//------ TELEMETRY BUFFER OUTPUT---------------------                                                                  
#warning concat("CLOCK ", str(clock,5,5))

#warning concat("DISTANCE_TO_ATMOSPHERE_m ", str(height_atm,5,5))
#warning concat("RADAR_ALTITUDE_m ", str(radar_alt,5,5))

#warning concat("ORBIT ROTATION DELTA ", str(amt2,5,5))
#warning concat("ORBIT ROTATION INTEGRAL ", str(amt[0],5,5))
                                                 
#warning concat("surface_pos x ", str(surface_pos.x,5,5))
#warning concat("surface_pos y ", str(surface_pos.y,5,5))
#warning concat("surface_pos z ", str(surface_pos.z,5,5))

#warning concat("orbitProbe x ", str(orbit.x,5,5))                                                                         
#warning concat("orbitProbe y ", str(orbit.y,5,5))
#warning concat("orbitProbe z ", str(orbit.z,5,5))

//#warning concat("orbit2 x ", str(orbit2.x,5,5))
//#warning concat("orbit2 y ", str(orbit2.y,5,5))
//#warning concat("orbit2 z ", str(orbit2.z,5,5))

//#warning concat("prograde x ", str(prograde.x,5,5))
//#warning concat("prograde y ", str(prograde.y,5,5))
//#warning concat("prograde z ", str(prograde.z,5,5))

#warning concat("orbitShield x ", str(shieldpos.x,5,5))                                                                         
#warning concat("orbitShield y ", str(shieldpos.y,5,5))
#warning concat("orbitShield z ", str(shieldpos.z,5,5))

#warning concat("DISTANCE_TO_SHIELD ", str(dst_shield,5,5))
  

//********************************* PROBE CAMERA *******************************************
                                                    
camera {
           location orbit // orbit position changing with clock using polar equations ( HEIGHT + ROTATION)
           look_at orbit2 // look "forward" or prograde                                                 
           sky vnormalize(orbit) // keep planet under the probe (lock the probe)
           angle 50 
       }
       
       
       
//********************************* STARFIELD ******************************************* 

sphere {
           <0,0,0>,500000
           hollow
           pigment {
                     image_map {
                                 png "starmap.png"
                                 map_type 1
                                 once
                               }
                   }
                   
           finish  {
                     ambient 20000000
                     diffuse 0.8                                                     
                            }
           scale 15000000
      }                                                                         
      
//--------EOF------------